require 'studio_game/game'
require 'studio_game/die'

require 'csv'

module StudioGame
  describe Game do
    before do
      @game = Game.new("Knuckleheads")

      @initial_health = 100
      @player = Player.new("moe", @initial_health)
      @player2 = Player.new("larry", 90)

      @game.add_player(@player)
      @game.add_player(@player2)
      $stdout = StringIO.new
    end

    it "assigns a treasure for points during a player's turn" do
      @game.play(1)

      expect(@player.points).not_to be_zero
    end

    it "w00ts the player when a high number is rolled" do
      allow_any_instance_of(Die).to receive(:roll).and_return(5)

      @game.play(3)

      expect(@player.health).to eq(@initial_health + 15 * 3)
    end

    it "skips the player when a medium number is rolled" do
      allow_any_instance_of(Die).to receive(:roll).and_return(3)

      @game.play(3)

      expect(@player.health).to eq(@initial_health)
    end

    it "formats the high score entry" do
      string = @game.high_score_entry(@player)

      expect(string).to eq("#{@player.name.ljust(20, '.')} #{@player.score}")
    end

    it "blams the player when a lower number is rolled" do
      allow_any_instance_of(Die).to receive(:roll).and_return(1)

      @game.play(3)

      expect(@player.health).to eq(@initial_health - 10 * 3)
    end

    it "computs total points as the sum of all players points" do
      @player.found_treasure(Treasure.new(:hammer, 50))
      @player2.found_treasure(Treasure.new(:crowbar, 400))

      expect(@game.total_treasure_points).to eq(450)
    end
  end
end
