require 'studio_game/player'
require 'studio_game/treasure_trove'

module StudioGame
  describe Player do

    before do
      @initial_health = 150
      @player = Player.new("larry", @initial_health)
      $stdout = StringIO.new
    end

    it "has a capitalized name" do
      expect(@player.name).to eq("Larry")
    end

    it "has an initial health" do
      expect(@player.health).to eq(@initial_health)
    end

    it "yields each found treasure and its total points" do
      @player.found_treasure(Treasure.new(:hammer, 50))
      @player.found_treasure(Treasure.new(:hammer, 50))
      @player.found_treasure(Treasure.new(:skillet, 100))
      @player.found_treasure(Treasure.new(:skillet, 100))
      @player.found_treasure(Treasure.new(:broomstick, 200))

      yielded = []

      @player.each_found_treasure do |treasure|
        yielded << treasure
      end

      expect(yielded).to eq([
                             Treasure.new(:hammer, 100),
                             Treasure.new(:skillet, 200),
                             Treasure.new(:broomstick, 200)
                            ])
    end

    it "computes points as the sum of all found treasures" do
      expect(@player.points).to eq(0)

      @player.found_treasure(Treasure.new(:hammer, 50))

      expect(@player.points).to eq(50)

      @player.found_treasure(Treasure.new(:crowbar, 400))

      expect(@player.points).to eq(450)

      @player.found_treasure(Treasure.new(:hammer, 50))

      expect(@player.points).to eq(500)
    end

    it "has a string representation" do
      @player.found_treasure(Treasure.new(:hammer, 50))
      @player.found_treasure(Treasure.new(:crowbar, 400))

      expect(@player.to_s).to eq("I'm #{@player.name}, health = #{@player.health}, points = 450, and score = #{@player.score}.")
    end

    it "computes a score as the sum of its health and points" do
      @player.found_treasure(Treasure.new(:hammer, 50))
      @player.found_treasure(Treasure.new(:crowbar, 400))

      expect(@player.score).to eq(450 + @initial_health)
    end

    it "increases the health by 15 when w00ted" do
      @player.w00t

      expect(@player.health).to eq(@initial_health + 15)
    end

    it "decreases the health by 10 when blammed" do
      @player.blam

      expect(@player.health).to eq(@initial_health - 10)
    end

    context "player has an initial health higher than 100" do
      before do
        @initial_health = 150
        @player = Player.new("larry", @initial_health)
      end

      it "is strong" do
        @player.strong?

        expect(@player).to be_strong
      end
    end

    context "player has an initial health of 100" do
      before do
        @initial_health = 100
        @player = Player.new("larry", @initial_health)
      end

      it "is not strong" do
        @player.strong?

        expect(@player).not_to be_strong
      end
    end

    context "in a collection of players" do

      before do
        @player1 = Player.new("moe", 80)
        @player2 = Player.new("curly", 90)
        @player3 = Player.new("larry", 100)

        @players = [@player1, @player2, @player3]
      end

      it "is sorted by decreased order" do
        expect(@players.sort).to eq([@player3, @player2, @player1])
      end
    end
  end
end
